import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.*;

//import hypertree.HTNode;

/**
 * The HTFileNodeV16 implements an example of HTNode encapsulating a File.
 */

public class HTFileNodeV16  implements Serializable    {
    static final long serialVersionUID = 5057209059047390834L;
    String  id;
    String  name;
    String  hint;
    String  color;
    String  URL;
    String  textColor;
    boolean especialMark;
    String  nodeClientData; // Atributo especial
    boolean leafNode;
    boolean hasHint;
    boolean hasData;	// Existe item procurado
    boolean hasArc;     // Existe algum filho com item proocurado

    private Hashtable children = null; // the children of this node


  /* --- Constructor --- */

    /**
     * Constructor.
     *
     * @param file the File that encapsulated the Tree
     */

    public HTFileNodeV16() {
      children       = new Hashtable();
	id             = "";
    	name           = "";
    	hint           = "";
    	color          = "";
    	URL            = "";
    	textColor      = "";
    	especialMark   = false;
    	nodeClientData = "";
    	leafNode       = true;
      hasHint        = false;
	hasData        = false;
	hasArc         = false;
    }

    public HTFileNodeV16(HTFileNodeV16 node) {
      children       = new Hashtable();
	id             = node.getID();
    	name           = node.getName();
    	hint           = node.getHint();
    	color          = node.getColor();
    	URL            = node.getURL();
    	textColor      = node.getTextColor();
    	especialMark   = false;
    	nodeClientData = node.getNodeClientData();
	if (node.isLeafNode())
    		leafNode       = true;
	else
		leafNode       = false;
      hasHint        = false;
	hasData        = false;
	hasArc         = false;
    }


  /* --- Tree management --- */

    /**
     * Add child to the node.
     * 
     * @param child    the HTFileNodeV16 to add as a child
     */
    protected void addChild(HTFileNodeV16 child) {
        children.put(child.getID(), child);
    }

    protected void removeChild(String id) {
        children.remove(id);
    }


  /* --- HTNode --- */

    /**
     * Returns the children of this node in an Enumeration.
     * If this node is a file, return a empty Enumeration.
     * Else, return an Enumeration full with HTFileNodeV16.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration children() {
        	return children.elements();
    }

    /**
     * Returns the children of this node in an Enumeration.
     * If this node is a file, return a empty Enumeration.
     * Else, return an Enumeration full with HTFileNodeV16.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration childrenNode() {
        return children.elements();
    }


    /**
     * Returns true if this node is not a directory.
     *
     * @return    <CODE>false</CODE> if this node is a directory;
     *            <CODE>true</CODE> otherwise
     */
    public boolean isLeaf() {
        return (leafNode);
    }
    
    public boolean isLeafNode() {
        return (leafNode);
    }


    public void setLeafNode(boolean t)
    {
		leafNode = t;
    }


    /**
     * Returns the attributes id of the node.
     * @return the id of the node 
     */
    public String getID() {
        return id;
    }

    
    /**
     * Returns the attributes name of the node.
     * @return the name of the node 
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the attributes hint of the node.
     * @return the hint of the node 
     */
    public String getHint() {
	  return hint;
    }

    /**
     * Returns the attributes URL of the node.
     * @return the URL of the node 
     */
    public String getURL() {
	  return URL;
    }

    /**
     * Returns the attributes COLOR of the node.
     * @return the COLOR of the node 
     */
    public String getColor() {
	  return color;
    }

    /**
     * Returns the attributes especial Mark of the node.
     * @return the especialMark of the node 
     */
    public boolean getEspecialMark() {
	  return especialMark;
    }

    /**
     * Returns the attributes text color of the node.
     * @return the text color of the node 
     */
    public String getTextColor() {
	  return textColor;
    }

    /**
     * Returns the attributes clienteData of the node.
     * @return the client data of the node 
     */
    public String getNodeClientData() {

	  if ((nodeClientData == null) || (nodeClientData.length()==0))
		return("10");
	  else
	  	return nodeClientData;

    }

    /**
     * Returns the attributes hasHint of the node.
     * @return boolean 
     */
    public boolean getHasHint() {
	  return hasHint;
    }

    /**
     * Returns the attributes hasData of the node.
     * @return boolean 
     */
    public boolean getHasData() {
	  return hasData;
    }

    /**
     * Returns the attributes hasArc of the node.
     * @return boolean 
     */
    public boolean getHasArc() {
	  return hasArc;
    }


    /**
     * Set the attributes id of the node.
     * @void
     */
    public void setID(String v) {
        id = v;
    }


    /**
     * Set the attributes name of the node.
     * @void
     */
    public void setName(String v) {
        name = v;
    }

    /**
     * Set the attributes Hint of the node.
     * @void
     */
    public void setHint(String v) {
	  hint = v;
    }

    /**
     * Set the attributes URL of the node.
     * @void
     */
    public void setURL(String v) {
	  URL = v;
    }

    /**
     * Set the attributes COLOR of the node.
     * @void 
     */
    public void setColor(String v) {
	  color = v;
    }

    /**
     * Set the attributes especial Mark of the node.
     * @void
     */
    public void setEspecialMark(boolean v) {
	  especialMark = v ;
    }
    /**
     * Set the attributes text color of the node.
     * @void
     */
    public void setTextColor(String v) {
	  textColor = v ;
    }

    /**
     * Set the attributes node Cliente data of the node.
     * @void
     */
    public void setNodeClientData(String v) {
	  nodeClientData = v ;
    }

    /**
     * Set the attributes hint of the node.
     * @void
     */
    public void setHasHint(boolean v) {
	  hasHint = v ;
    }

    /**
     * Set the attributes hasData of the node.
     * @void
     */
    public void setHasData(boolean v) {
	  hasData = v ;
    }

    /**
     * Set the attributes hasArc of the node.
     * @void
     */
    public void setHasArc(boolean v) {
	  hasArc = v ;
    }

    /**
     * Insert a new child on actual node
     * @void
     */

    public String editFilho()
    {
		String texto="";

		if (this.getName().length() > 0)
			texto = "Nome:"+this.getName();
		if (this.getColor().length() > 0)
			texto = texto + ";Cor:"+this.getColor();
		if (this.getTextColor().length() > 0)
			texto = texto + ";Cor_texto:"+this.getTextColor();
		if (this.getHint().length() > 0)
			texto = texto + ";Hint:"+this.getHint();
		if (this.getURL().length() > 0)
			texto = texto + ";URL:"+this.getURL();

		return texto;

    }

    public void saveFilho(String texto)
    {

		// Esta funo ganhou funcionalidades extras para reduzir o nmero de atualizaes de	
		// rvore j geradas.

		String nome="";
		String cor="";
		String hint_A="";
		String URL_A="";
		String cor_t="";

		int inx;

		inx = texto.toLowerCase().indexOf("mudarNome:");


//		inx = texto.toLowerCase().indexOf("ordenar:");


		if (inx >= 0) { // Foi pedido para alterar nome do n filho
			//this.setNodeClientData("1");
			//this.setID(this.getID()+"_1");

		}
		else {

			inx = texto.toLowerCase().indexOf("nome:");

			if (inx >= 0)
			for (int i=inx+5; i < texto.length(); i++) {			
				if (texto.charAt(i) == ';')
					break;
				else
					nome = nome + texto.charAt(i) ;
			}
			inx = texto.toLowerCase().indexOf("cor:");
			if (inx >= 0)
			for (int i=inx+4; i < texto.length(); i++) {			
				if (texto.charAt(i)  == ';')
					break;
				else
					cor = cor + texto.charAt(i) ;
			}
			inx = texto.toLowerCase().indexOf("hint:");
			if (inx >= 0)
			for (int i=inx+5; i < texto.length(); i++) {			
				if (texto.charAt(i)  == ';')
					break;
				else
					hint_A = hint_A + texto.charAt(i) ;
			}

			inx = texto.toLowerCase().indexOf("url:");
			if (inx >= 0)
			for (int i=inx+4; i < texto.length(); i++) {			
				if (texto.charAt(i)  == ';')
					break;
				else
					URL_A = URL_A + texto.charAt(i) ;
			}

			inx = texto.toLowerCase().indexOf("cor_texto:");
			if (inx >= 0)
			for (int i=inx+10; i < texto.length(); i++) {			
				if (texto.charAt(i)  == ';')
					break;
				else
					cor_t = cor_t + texto.charAt(i) ;
			}

			this.setName(nome);
			this.setColor(cor);
			this.setHint(hint_A);
			this.setURL(URL_A);

			if (cor_t.length() >= 8)
				this.setTextColor(cor_t);
		}

}



    public void insereFilho(int nchilds) {

            String textColorDefault= "0xFFFFFF";
	      String nodeColorDefault= "0x00FF00"; 

		HTFileNodeV16 node = new HTFileNodeV16();

		Date data = new Date();

	
		node.setID("_"+data.getTime()+"_F");

/*
		if (nchilds+1 < 10)
			node.setID("_"+"00000"+nchilds+1+"_F");
		if ((nchilds+1 >= 10) && (nchilds+1 < 100))
			node.setID("_"+"0000"+nchilds+1+"_F");
		if ((nchilds+1 >= 100) && (nchilds+1 < 1000))
			node.setID("_"+"000"+nchilds+1+"_F");
		if ((nchilds+1 >= 1000) && (nchilds+1 < 10000))
			node.setID("_"+"00"+nchilds+1+"_F");
		if ((nchilds+1 >= 10000) && (nchilds+1 < 100000))
			node.setID("_"+"0"+nchilds+1+"_F");
*/
		node.setColor(this.getColor());//nodeColorDefault);
		node.setTextColor(this.getTextColor() );
		node.setName("Filho "+ node.getID());
		node.setLeafNode(true);	

		this.setLeafNode(false);
		this.setHasHint(false);
		this.setEspecialMark(false);		
		this.addChild(node);	  
    }

    public void deleteFilho(String id) {
		this.removeChild(id);	  
    }

    public void insereRamo(HTNode ramo, int nNodes) {	

		if (ramo != null && nNodes > -1)	
			this.insereFromRamo((HTFileNodeV16) ramo, this, nNodes, true );

		if (ramo != null && nNodes == -1) {	// Criando espelho
			HTFileNodeV16 node = new HTFileNodeV16((HTFileNodeV16) ramo);
			node.setID(node.getID()+"_M_"+this.getID());
			node = (HTFileNodeV16) ramo;
			this.addChild(node);
			this.setLeafNode(false);
		}

		if (ramo != null && nNodes == -2) {	// Criando link escondido			
			HTFileNodeV16 node = new HTFileNodeV16((HTFileNodeV16) ramo);
			node.setID(node.getID()+"_H_"+this.getID());
			nNodes++;
			node.setHint("Representa um ciclo para o n: " + node.getName() + " &crlf " + node.getHint());
			this.addChild(node);
			this.setLeafNode(false);
		}			
    }

    public void insereFromRamo(HTFileNodeV16 origem, HTFileNodeV16 destino, int nNodes, boolean fl) {

		HTFileNodeV16 node1; 

		if ((origem == null) || (destino==null) || (destino == origem))
			return;

		if (fl) {
			node1 = new HTFileNodeV16(origem);
			//node = origem;
			Date data = new Date();
			node1.setID("_"+origem.getID()+"_"+data.getTime()+"_F");
			nNodes++;
			node1.setColor(destino.getColor());
			destino.addChild(node1);
		}
		else
			node1 = destino;

		destino.setLeafNode(false);
		

		for (Enumeration e=origem.children(); e.hasMoreElements(); ) {
            	HTFileNodeV16 childNode = (HTFileNodeV16) e.nextElement();
            	if (!childNode.isLeaf()) {
				HTFileNodeV16 node = new HTFileNodeV16(childNode);
				//node = childNode;
				Date data = new Date();
				node.setID("_"+childNode.getID()+"_"+data.getTime()+"_F");
				nNodes++;
				//node.setTextColor(childNode.getTextColor());
				//node.setHint(childNode.getHint());
				//node.setURL(childNode.getURL());

				node.setColor(node1.getColor());
				node1.setLeafNode(false);
				node1.addChild(node);

				insereFromRamo(childNode, node, nNodes, false);
            	}
			else {
				HTFileNodeV16 node = new HTFileNodeV16(childNode);
				//node = childNode;
				Date data = new Date();
				node.setID("_"+childNode.getID()+"_"+data.getTime()+"_F");
				nNodes++;
				node.setColor(node1.getColor());

				//node.setTextColor(childNode.getTextColor());
				//node.setHint(childNode.getHint());
				//node.setURL(childNode.getURL());

				node1.setLeafNode(false);
				node1.addChild(node);
			}
		}
    }


}

