/*
 * HTGeodesic.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

// Cdigo alterado por Silvio Evangelista

//package hypertree;
import java.awt.*;//Graphics;
//import java.awt.Font;
//import java.awt.Color;
//import java.awt.Insets;

import java.util.*;
import javax.swing.*;

/**
 * The HTDraw class implements the drawing model for the HTView.
 */
class HTDraw {

    private final static int NBR_FRAMES = 10; // number of intermediates animation frames
    private HTModel model = null;  // the tree model
    // Bruno - private
    public HTView view = null;  // the view using this drawing model
    private HTDrawNode drawRoot = null;  // the root of the drawing tree 
    private HTCoordS sOrigin = null;  // origin of the screen plane
    private HTCoordS sMax = null;  // max point in the screen plane 
    private double[] ray = null;
    private boolean fastMode = false; // fast mode
    private String ramo = null;  // Ramo a ser copiado com o CONTROL V
    private static int fh,  fa,  fl;
    private static int charWidth;
    private static FontMetrics fm;
    static Font mainFont;
    /*
    static String id1="";
    static String id2="";    
    static boolean processa=false;*/
    /*
    int     hpx = 25;
    int     hpy = 50;
    int     hw  = 10;
    int     hl  = 15; 
    int     hs  = 30;
     */
    /* --- Constructor --- */

    /**
     * Constructor.
     *
     * @param model    the tree model to draw 
     * @param view     the view using this drawing model
     */
    HTDraw(HTModel model, HTView view) {

        this.view = view;
        this.model = model;

        HTModelNode root = model.getRoot();
        sOrigin = new HTCoordS();
        sMax = new HTCoordS();

        ray = new double[4];
        ray[0] = model.getLength();

        for (int i = 1; i < ray.length; i++) {
            ray[i] = (ray[0] + ray[i - 1]) / (1 + (ray[0] * ray[i - 1]));
        }

        if (root.isLeaf()) {
            drawRoot = new HTDrawNode(null, root, this);
        } else {
            drawRoot = new HTDrawNodeComposite(null, (HTModelNodeComposite) root, this);
        }
    }


    /* --- Screen coordinates --- */
    /**
     * Refresh the screen coordinates of the drawing tree.
     */
    void refreshScreenCoordinates() {
        Insets insets = view.getInsets();
        sMax.x = (view.getSize().width - insets.left - insets.right) / 2;
        sMax.y = (view.getSize().height - insets.top - insets.bottom) / 2;
        sOrigin.x = sMax.x + insets.left;
        sOrigin.y = sMax.y + insets.top;
        drawRoot.refreshScreenCoordinates(sOrigin, sMax);
    }

    /**
     * Returns the origin of the screen plane.
     * WARNING : this is not a copy but the original object.
     *
     * @return    the origin of the screen plane
     */
    HTCoordS getSOrigin() {
        return sOrigin;
    }

    /**
     * Return the point representing the up right corner
     * of the screen plane, thus giving x and y maxima.
     * WARNING : this is not a copy but the original object.
     *
     * @return    the max point
     */
    HTCoordS getSMax() {
        return sMax;
    }

    /* --- Drawing --- */
    /**
     * Draws the branches of the hyperbolic tree.
     *
     * @param g    the graphic context
     */
    void drawBranches(Graphics g) {
        drawRoot.drawBranches(g);
    }

    /**
     * Draws the nodes of the hyperbolic tree.
     *
     * @param g    the graphic context
     */
    void drawNodes(Graphics g) {

        mainFont = new Font(this.view.getFontNode(), Font.PLAIN, this.view.getFontSizeNode());

        g.setFont(mainFont);

        fm = g.getFontMetrics();
        fh = fm.getHeight();
        fa = fm.getAscent();
        fl = fm.getLeading();

        String name = "a";
        char[] nameC = name.toCharArray();
        charWidth = fm.charsWidth(nameC, 0, 1);

        g.setColor(new Color(Color.decode(getLinkColor()).getRGB()));

        //g.drawString( "Busca:",  15, 14); 

        //g.drawString( " 2003 - HiperEditor -", 15,  getHeightPanel() - 20);

        //g.drawString( "Embrapa Informtica Agropecuria", 15,  getHeightPanel() - 6);


        //drawRoot.drawNodes0(g, true);
        String res = drawRoot.getNodeClientData();

        /* Bruno
         * Indica se os ns filhos devem mostrar as figuras
         */
		drawRoot.drawLinesGraph(g); // 12/08/2009
		
        if (res.charAt(0) == '1') {	// Mostra as figuras
            drawRoot.drawNodes0(g, true);
            drawRoot.drawNodes(g, true);
        } else {
            drawRoot.drawNodes0(g, false);
            drawRoot.drawNodes(g, false);	// Esconde as figuras
        }
	/*
	if (processa) {
		processa=false;
		System.out.println("****** " + id1 + " " + id2);
		view.acertaFuture(id1, id2);
	}
*/
		
    // Fim bruno

    //drawRoot.drawNodes(g);

    }

    int verifyHomeClick(HTCoordS zs) {
        /*
        int px = zs.x;
        int py = zs.y;
        Rectangle  rect = view.getHomeRect();
        
        if (  (px >= (rect.x)) && ( px<= (rect.x + rect.width)) && (py >= rect.y) && (py <= rect.y+rect.height)) {
        translateToRootOrigin();
        return 0;
        }
        
        if (view.getCenterNode() != null)			
        if (  (px >= (rect.x+rect.width)) && ( px<= (rect.x+rect.width+ rect.width)) && (py >= rect.y) && (py <= rect.y+rect.height)) {
        
        HTDrawNode result = findNode(view.getCenterNode().toLowerCase());
        if (result != null) {
        translateToOrigin(result);	
        }			
        return 0;
        }
        
        if (  (px >= (view.getSize().width-rect.width*2+rect.x)) && ( px<= (view.getSize().width-rect.width*2+rect.x+rect.width)) && (py >= 10) && (py <= 10+rect.height)) {
        return 1;			
        }
         */

        return 0;
    }

    void goOrigem() {
        translateToRootOrigin();
    }

    boolean verifyOKClick(HTCoordS zs) {

        int px = zs.x;
        int py = zs.y;
        Rectangle rect = view.getOKRect();

        if (rect != null) {
            if ((px >= (rect.x)) && (px <= (rect.x + rect.width)) && (py >= rect.y) && (py <= rect.y + rect.height)) {
                return (true);
            }
        }
        return false;
    }

    boolean verifyCANCELClick(HTCoordS zs) {

        int px = zs.x;
        int py = zs.y;
        Rectangle rect = view.getCANCELRect();

        if (rect != null) {
            if ((px >= (rect.x)) && (px <= (rect.x + rect.width)) && (py >= rect.y) && (py <= rect.y + rect.height)) {
                return (true);
            }
        }
        return false;
    }

    boolean verifyAplicarClick(HTCoordS zs) {

        int px = zs.x;
        int py = zs.y;
        Rectangle rect = view.getAplicarRect();

        if (rect != null) {
            if ((px >= (rect.x)) && (px <= (rect.x + rect.width)) && (py >= rect.y) && (py <= rect.y + rect.height)) {
                return (true);
            }
        }
        return false;
    }

    /**
     * Draws the Hint of the selectioned node.
     *
     * @param g    the graphic context
     */
    void drawHint(Graphics g) {
        drawRoot.drawHint(g);
    }


    /* --- Translation --- */
    /**
     * Translates the hyperbolic tree by the given vector.
     *
     * @param t    the translation vector
     */
    void translate(HTCoordE zs, HTCoordE ze) {
        HTCoordE zo = new HTCoordE(drawRoot.getOldCoordinates());
        zo.x = -zo.x;
        zo.y = -zo.y;
        HTCoordE zs2 = new HTCoordE(zs);
        zs2.translate(zo);

        HTCoordE t = new HTCoordE();
        double de = ze.d2();
        double ds = zs2.d2();
        double dd = 1.0 - de * ds;
        t.x = (ze.x * (1.0 - ds) - zs2.x * (1.0 - de)) / dd;
        t.y = (ze.y * (1.0 - ds) - zs2.y * (1.0 - de)) / dd;

        if (t.isValid()) {
            HTTransformation to = new HTTransformation();
            to.composition(zo, t);

            drawRoot.transform(to);
            view.repaint();
        }
    }

    /**
     * Signal that the translation ended.
     */
    void endTranslation() {
        drawRoot.endTranslation();
    }

    /**
     * Translate the hyperbolic tree so that the given node 
     * is put at the origin of the hyperbolic tree.
     *
     * @param node    the given HTDrawNode
     */
    void translateToOrigin(HTDrawNode node) {
        view.stopMouseListening();
        AnimThread t = new AnimThread(node);
        t.start();
    }

    void translateToRootOrigin() {
        view.stopMouseListening();
        AnimThread t = new AnimThread(drawRoot);
        t.start();
    }

    /**
     * Restores the hyperbolic tree to its origin.
     */
    void restore() {
        drawRoot.restore();
        view.repaint();
    }

    /**
     * reDraw the hyperbolic tree.
     * This functions is called when the user ask for a hint
     */
    void reDraw() {
        //view.setHasHint(true);	      
        view.repaint();
    //view.setHasHint(false);
    }

    /**
     * reDraw the hyperbolic tree.
     * This functions is called when the user ask for a hint
     */
    void reDraw(String v) {
        //view.setHasHint(true);	      
        view.setActualNode(v);
        view.repaint();
    //view.setHasHint(false);
    }

    /**
     * Get width of the panel.
     * 
     */
    int getWidthPanel() {
        return view.getSize().width; // getWidth();
    }

    /**
     * Get length of the Panel
     */
    int getHeightPanel() {
        return view.getSize().height; // getHeight();
    }

    /**
     * Set that there is one Hint to be showed
     */
    void setHint(boolean v) {
        view.setHasHint(v);
    }

    String getHelpFile() {
        return view.getHelpFile();
    }

    String getUrlAdr() {
        return view.getUrlAdr();
    }
    
    String getLiteAdr() {
	return view. getLiteAdr();
    }
    // 14/11/2005
    /**
     * get the representation type of search:
     *   line: line thicher
     *   blende: blende mode. This type use transparency color.
     *      */
    boolean getSearchType() {
        return view.getSearchType();
    }

    /**
     * Indicates if the hypertree space is in search mode.
     */
    boolean getIsSearch() {
        return view.getIsSearch();
    }

    // *************
    /**
     * Sets the fast mode, where nodes are no more drawed.
     *
     * @param mode    setting on or off.
     */
    void fastMode(boolean mode) {
        if (mode != fastMode) {
            fastMode = mode;
            drawRoot.fastMode(mode);
            if (mode == false) {
                view.repaint();
            }
        }
    }

    // 13/09/2004    
    int verifyFolder(HTCoordS zs) {
        return drawRoot.verifyFolder(zs);
    }

    // END

    /* --- Node searching --- */
    /**
     * Returns the node (if any) whose screen coordinates' zone
     * contains thoses given in parameters.
     *
     * @param zs    the given screen coordinate
     * @return      the searched HTDrawNode if found;
     *              <CODE>null</CODE> otherwise
     */
    HTDrawNode findNode(HTCoordS zs) {
        return drawRoot.findNode(zs);
    }

    /**
     * Searching a node in HTDrawNodeCompisite
     *
     * @param idNode String
     */
    HTDrawNode findNode(String idNode) {
        return (drawRoot.findNode(idNode));
    }
    /*
    void procFuture(String i1, String i2 ) {
	id1=i1;
	id2=i2;    
	processa=true;
    }*/

    boolean getClickOnImage() {
        return drawRoot.getClickOnImage();
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */
    String getHintColor() {
        return view.getHintColor();
    }
	
	String getGraphColor() {
        return view.getGraphColor();
    }

    /**
     * Returns the name of the font via model class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */
    String getFontNode() {
        return view.getFontNode();
    }

    /**
     * Returns the size of the font, via model class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */
    int getFontSizeNode() {
        return view.getFontSizeNode();
    }

    /**
     * Returns the border type of retangule armazende in view class
     * contains thoses given in parameters.
     *
     * @return      String 
     *              
     */
    String getBorderType() {
        return view.getBorderType();
    }

    /**
     * Returns the text of search box, if has one.
     *
     *
     * @return      String 
     *              
     */
    String getSearchWord() {
        return (view.getSearchWord());
    }

    String getTextField() {
        return (view.getTextField());
    }

    String getArestaField() {
        return (view.getArestaField());
    }

    String getTexWord() {
        return (view.getTexWord());
    }

    String getCorWord() {

        String texto = view.getCorWord();

        if (texto.trim().length() < 6) {
            texto = "0x555555";
        }
        if (texto.trim().length() > 8) {
            texto = texto.substring(0, 7);
        }
        texto = texto.toLowerCase();

        if (texto.trim().length() == 8) {
            texto = "0x" + texto.substring(2);
        }

        if (texto.trim().length() == 7) {
            texto = "0x" + texto.substring(1);
        }

        if (texto.trim().length() == 6) {
            texto = "0x" + texto;
        }

        for (int i = 2; i < texto.trim().length(); i++) {
            if (texto.charAt(i) != '0' && texto.charAt(i) != '1' && texto.charAt(i) != '2' && texto.charAt(i) != '3' && texto.charAt(i) != '4' && texto.charAt(i) != '5' && texto.charAt(i) != '6' && texto.charAt(i) != '7' && texto.charAt(i) != '8' && texto.charAt(i) != '9' && texto.charAt(i) != 'a' && texto.charAt(i) != 'b' && texto.charAt(i) != 'c' && texto.charAt(i) != 'd' && texto.charAt(i) != 'e' && texto.charAt(i) != 'f') {
                texto = texto.replace(texto.charAt(i), '5');
            }
        }

        if (texto.trim().length() == 8) {
            texto = "0x" + texto.substring(2);
        }

        view.setCorWord(texto);

        return (texto);
    }

    String getCorTextWord() {

        String texto = view.getCorTextWord();

        if (texto.trim().length() < 6) {
            texto = "0x555555";
        }
        if (texto.trim().length() > 8) {
            texto = texto.substring(0, 7);
        }
        texto = texto.toLowerCase();

        if (texto.trim().length() == 8) {
            texto = "0x" + texto.substring(2);
        }

        if (texto.trim().length() == 7) {
            texto = "0x" + texto.substring(1);
        }

        if (texto.trim().length() == 6) {
            texto = "0x" + texto;
        }

        for (int i = 2; i < texto.trim().length(); i++) {
            if (texto.charAt(i) != '0' && texto.charAt(i) != '1' && texto.charAt(i) != '2' && texto.charAt(i) != '3' && texto.charAt(i) != '4' && texto.charAt(i) != '5' && texto.charAt(i) != '6' && texto.charAt(i) != '7' && texto.charAt(i) != '8' && texto.charAt(i) != '9' && texto.charAt(i) != 'a' && texto.charAt(i) != 'b' && texto.charAt(i) != 'c' && texto.charAt(i) != 'd' && texto.charAt(i) != 'e' && texto.charAt(i) != 'f') {
                texto = texto.replace(texto.charAt(i), '5');
            }
        }

        if (texto.trim().length() == 8) {
            texto = "0x" + texto.substring(2);
        }

        view.setCorTextWord(texto);

        return (texto);
    }

    String getNameArest() {
        return (view.getNameArest());
    }

    String getHintWord() {
        return (view.getHintWord());
    }

    String getURLWord() {
        return (view.getURLWord());
    }

    String getBackColor() {
        return view.getBackColor();
    }

    // 17/11/2005
    int getBackRed() {

        return view.getBackRed();

    }

    int getBackGreen() {

        return view.getBackGreen();

    }

    int getBackBlue() {

        return view.getBackBlue();

    }

// ******

// ******
    /**
     * Returns the link color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */
    String getLinkColor() {
        return view.getLinkColor();
    }

    /**
     * Returns the search color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */
    String getSearchColor() {
        return view.getSearchColor();
    }

    void setSearchWord(String texto) {
        view.setSearchWord(texto);
    }

    void setTextWord(String texto) {
        view.setTextWord(texto);
    }

    void setArestaTextoField(String texto) {
        view.setArestaTextoField(texto);
    }

    void setCorWord(String texto) {


        view.setCorWord(texto);
    }

    void setCorTextWord(String texto) {
        view.setCorTextWord(texto);
    }

    void setHintWord(String texto) {
        view.setHintWord(texto);
    }

    void setURLWord(String texto) {
        view.setURLWord(texto);
    }

    int getCharWidth() {
        return (charWidth);
    }

    int getCharHeight() {
        return (fh);
    }

    int getAscent() {
        return (fa);
    }

    int getLeading() {
        return (fl);
    }

    int getStringWidth(char[] nameC, int init, int end) {
        return (fm.charsWidth(nameC, init, end));
    }

    int getStringWidth(String nameC) {
        return (fm.stringWidth(nameC));
    }

    public String getRamo() {
        return (ramo);
    }

    public void setRamo(String v) {
        ramo = v;
    }

    public void setEditedChild(boolean v) {
        model.setEditedChild(v);
    }

    public boolean getEditedChild() {
        return model.getEditedChild();
    }

    Button getCorT() {
        return (view.getCorT());
    }

    Button getCorF() {
        return (view.getCorF());
    }

    Button getAplicarT() {
        return (view.getAplicarT());
    }

    Button getAplicarF() {
        return (view.getAplicarF());
    }

    Button getAplicarI() {
        return (view.getAplicarI());
    }

    Button getAplicarRI() {
        return (view.getAplicarRI());
    }

    Button getOk() {
        return (view.getOk());
    }

    Button getCancel() {
        return (view.getCancel());
    }
    //Bruno
    Button getLinks() {
        return (view.getLinks());
    }

    Button getFormas() {
        return (view.getFormas());
    }

    JCheckBox getCheckBox() {
        return (view.getCheckBox());
    }
    
    JCheckBox getCheckBox1() {
        return (view.getCheckBox1());
    }
    
    JButton getMesclar() {
        return (view.getMesclar());
    }


    JButton getDeletar() {
        return (view.getDeletar());
    }

    JButton getInserir() {
        return (view.getInserir());
    }

    JButton getEditar() {
        return (view.getEditar());
    }

    JButton getCopiar() {
        return (view.getCopiar());
    }

    JButton getColar() {
        return (view.getColar());
    }
    JButton getColarED() {
        return (view.getColarED());
    }

    JButton getCentralizar() {
        return (view.getCentralizar());
    }

    JButton getGravar() {
        return (view.getGravar());
    }

    JButton getVerificarLinks() {
        return (view.getVerificarLinks());
    }

    JButton getFolderTree() {
        return (view.getFolderTree());
    }

    JButton getConfigNavegador() {
        return (view.getConfigNavegador());
    }

    JButton getWeb() {
        return (view.getWeb());
    }

    JButton getGravarXML() {
        return (view.getGravarXML());
    }

    JButton getRecuperar() {
        return (view.getRecuperar());
    }

    JButton getColarEscondido() {
        return (view.getColarEscondido());
    }

    public JFrame getViewFrame() {
        return view.getViewFrame();
    }

    public void setViewFrameCursor(int n) {
        view.setViewFrameCursor(n);
    }
    
    public void setViewLineGraph(boolean n) {
        view.setViewLineGraph(n);
    }
    
    public boolean getViewLineGraph() {
        return view.getViewLineGraph();
    }

    TextField getCorFieldF() {
        return (view.getCorFieldF());
    }

    TextField getCorTextF() {
        return (view.getCorTextF());
    }

    public void undoGrava() {
        view.undoGrava();
    }

    public void undoRecupera() {
        view.undoRecupera();
    }

    public void grava() {
        view.grava();
    }
    // Silvio 19/09/2006
    public void verificaLinks() {
        view.verificaLinks();
    }

    public void folderTree() {
        view.folderTree();
    }

    public void configNavegador() {
        view.configNavegador();
    }

    public void webCrawling(String id) {
        view.webCrawling(id);
    }

    public boolean getHasUndo() {
        return (view.getHasUndo());
    }


    /* --- Inner animation thread --- */
    /**
     * The AnimThread class implements the thread that do the animation
     * when clicking on a node.
     */
    class AnimThread
            extends Thread {

        private HTDrawNode node = null; // node to put at the origin
        private Runnable tTask = null; // translation task

        /**
         * Constructor.
         *
         * @param node    the node to put at the origin
         */
        AnimThread(HTDrawNode node) {
            this.node = node;
        }

        /**
         * Do the animation.
         */
        public void run() {
            HTCoordE zn = node.getOldCoordinates();
            HTCoordE zf = new HTCoordE();

            int frames = NBR_FRAMES;
            int nodes = model.getNumberOfNodes();

            double d = zn.d();
            for (int i = 0; i < ray.length; i++) {
                if (d > ray[i]) {
                    frames += NBR_FRAMES / 2;
                }
            }

            double factorX = zn.x / frames;
            double factorY = zn.y / frames;

            for (int i = 1; i < frames; i++) {
                zf.x = zn.x - (i * factorX);
                zf.y = zn.y - (i * factorY);
                tTask = new TranslateThread(zn, zf);
                try {

                    //SwingUtilities.invokeAndWait(tTask);
                    sleep(10);
                    tTask.run();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            zf.x = 0.0;
            zf.y = 0.0;
            tTask = new LastTranslateThread(zn, zf);
            try {
                //SwingUtilities.invokeAndWait(tTask);
                sleep(25);
                tTask.run();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        /* --- Inner's inner --- */
        class TranslateThread
                implements Runnable {

            HTCoordE zStart = null;
            HTCoordE zEnd = null;

            TranslateThread(HTCoordE z1, HTCoordE z2) {
                zStart = z1;
                zEnd = z2;
            }

            public void run() {
                translate(zStart, zEnd);
                view.repaint();
            }
        }

        class LastTranslateThread
                implements Runnable {

            HTCoordE zStart = null;
            HTCoordE zEnd = null;

            LastTranslateThread(HTCoordE z1, HTCoordE z2) {
                zStart = z1;
                zEnd = z2;
            }

            public void run() {
                translate(zStart, zEnd);
                endTranslation();
                view.repaint();
                view.startMouseListening();
            }
        }
    }

    int getNumAnds() {
        return (view.getNumAnds());
    }

    String getWordI(int i) {
        return (view.getWordI(i));
    }

    public void setCenter(String v) {
        model.setCenter(v);
    }
    // Silvio: 29/09/2004
    void executaSearchWord(String search) {
        view.executaSearchWord(search);
    }
    // Silvio
    String getPath() {
        return view.getPath();
    }
    // Bruno - Poligono
    public void setIndicePoligono(int p) {
        model.setIndicePoligono(p);
    }

    public int getIndicePoligono() {
        return model.getIndicePoligono();
    }

    public int getRectx(int i, int largura, int posicaox) {
        return view.getRectx(i, largura, posicaox);
    }

    public int getRecty(int i, int largura, int posicaoy) {
        return view.getRecty(i, largura, posicaoy);
    }

    int getWidth(int i, int largura) {
        return view.getWidth(i, largura);
    }

    int getHeight(int i, int largura) {
        return view.getHeight(i, largura);
    }

    String getName() {
        return view.getName();
    }
    // Poligono
    public Polygon getPoligono(int indice, int posicaox, int posicaoy, int largura) {
        return view.getPoligono(indice, posicaox, posicaoy, largura);
    }

    public void mostraDlgFormas(HTDrawNode node) {
        view.mostraDlgFormas(node);
    }

    public void  addGraphLine(HTDrawNode nodeClicked, HTDrawNode nodeClickedA, boolean dir) {
	view.addGraphLine(nodeClicked, nodeClickedA, dir);
    }

int nGraphLines() {
	return view.nGraphLines();

}

Point getGraphNode1(int i) {
	return view.getGraphNode1(i);
	
}

Point getGraphNode2(int i) {
	return view.getGraphNode2(i);
	
}

void removeGraphLine(HTDrawNode n1) {
	view.removeGraphLine(n1);
}

void setArestaLineGraph(String ID, String nome) {
	view.setArestaLineGraph(ID, nome);
}

String getArcName(int i) {
	return view.getArcName(i);	
}

void folderFechadoGraphLine(HTDrawNode node) {    
	view.folderFechadoGraphLine(node);    
}

 public  boolean mostraGraph(int i) {
		return view.mostraGraph(i);
	}


        public  boolean getDirecionado(int i) {
		return view.getDirecionado(i);
	}

	
	
	public void acertaGraphLine(HTNode node) {
		view.acertaGraphLine(node);
	}
	
	HTDrawNode getRoot() {
		return (HTDrawNode) drawRoot;
	}

	void loadTreeAt(HTDrawNode nodeClicked, String filename) {
		view.loadTreeAt(nodeClicked, filename);
	}
	


}

