/*
 * HTGeodesic.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

// Cdigo alterado por Silvio Evangelista

//package hypertree;
/**
 * The HTModel class implements the model for the HyperTree.
 * It's a tree of HTModelNode and HTModelNodeComposite, each keeping the
 * initial layout of the tree in the Poincarre's Model.
 */
class HTModel {

    private HTModelNode root = null; // the root of the tree's model 
    private double length = 0.3;  // distance between node and children
    private int nodes = 0;    // number of nodes
    private boolean inseredChild = false;
    private boolean editedChild = false;
    private boolean isSearchText = false; // 14/11/2005
    private String textoEdicao; // Texto de edicao
    private String centerNode = null;// centro da insero


    /* --- Constructor --- */
    /**
     * Constructor.
     *
     * @param root    the root of the real tree 
     */
    HTModel(HTNode root) {
        if (root.isLeaf()) {
            this.root = new HTModelNode(root, this);
        } else {
            this.root = new HTModelNodeComposite(root, this);
        }
        this.root.layoutHyperbolicTree();
    }

    public void setNewModel(HTNode root) {
        if (root.isLeaf()) {
            this.root = new HTModelNode(root, this);
        } else {
            this.root = new HTModelNodeComposite(root, this);
        }
        this.root.layoutHyperbolicTree();
    }

    public void setInseredChild(boolean v) {
        inseredChild = v;
    }

    public void setCenter(String v) {
        centerNode = v;
    }

    public String getCenter() {
        return centerNode;
    }

    public boolean getInseredChild() {
        return (inseredChild);
    }

    public void setEditedChild(boolean v) {
        editedChild = v;
    }

    public boolean getEditedChild() {
        return (editedChild);
    }
    // 14/11/2005
    public boolean getIsSearchText() {
        return (isSearchText);
    }

    public void setIsSearchText(boolean v) {
        isSearchText = v;
    }

    // ******
    /* --- Accessor --- */
    /**
     * Returns the root of the tree model.
     *
     * @return    the root of the tree model
     */
    HTModelNode getRoot() {
        return root;
    }

    /**
     * Returns the distance between a node and its children
     * in the hyperbolic space.
     *
     * @return    the distance
     */
    double getLength() {
        return length;
    }


    /* --- Number of nodes --- */
    /**
     * Increments the number of nodes.
     */
    void incrementNumberOfNodes() {
        nodes++;
    }

    /**
     * Returns the number of nodes.
     *
     * @return    the number of nodes
     */
    int getNumberOfNodes() {
        return nodes;
    }

    void setTextoEdicao(String t) {
        textoEdicao = t;
    }

    String getTextoEdicao() {
        return (textoEdicao);
    }
    //Poligono
    public void setIndicePoligono(int p) {
        root.setIndicePoligono(p);
    }

    public int getIndicePoligono() {
        return root.getIndicePoligono();
    }
}

