/*
 * HTGeodesic.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

// Cdigo alterado por Silvio Evangelista

//package hypertree;
/**
 * The HTTransformation class implements a isometrie transformation
 * in the hyperbolic space.
 */
class HTTransformation {

    HTCoordE P = null; // translation vector
    HTCoordE O = null; // rotation vector 

    /* --- Constructor --- */
    /**
     * Constructor.
     */
    HTTransformation() {
        P = new HTCoordE();
        O = new HTCoordE();
    }

    /* --- Composition of 2 translation --- */
    /**
     * Compose the 2 given vectors translations 
     * into one given transformation.
     */
    void composition(HTCoordE first, HTCoordE second) {
        P.x = first.x + second.x;
        P.y = first.y + second.y;

        HTCoordE d = new HTCoordE(second);
        d.y = -d.y;
        d.multiply(first);
        d.x += 1;
        P.divide(d);

        O.x = first.x;
        O.y = -first.y;
        O.multiply(second);
        O.x += 1;
        O.divide(d);
    }

    /* --- ToString --- */
    /**
     * Returns a string representation of the object.
     *
     * @return    a String representation of the object
     */
    public String toString() {
        String result = "Transformation : " +
                "\n\tP = " + P +
                "\n\tO = " + O;
        return result;
    }
}
