//
// This is a small copy of the Java 2 library class to make the software works with 1.1 Java
//
/*
 * @(#)URLDecoder.java	1.21 02/04/15
 * Utility class for HTML form decoding. This class contains static methods
 * for decoding a String from the <CODE>application/x-www-form-urlencoded</CODE>
 * MIME format.
 * @author  Mark Chamness
 * @author  Michael McCloskey
 * @version 1.21, 04/15/02
 * @since   1.2
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

import java.io.UnsupportedEncodingException;

public class URLDecoder {

    /**
     * Decodes a <code>application/x-www-form-urlencoded</code> string using a specific
     * encoding scheme.
     * The supplied encoding is used to determine
     * what characters are represented by any consecutive sequences of the
     * form "<code>%<i>xy</i></code>".
     * <p>
     * <em><strong>Note:</strong> The <a href=
     * "http://www.w3.org/TR/html40/appendix/notes.html#non-ascii-chars">
     * World Wide Web Consortium Recommendation</a> states that
     * UTF-8 should be used. Not doing so may introduce
     * incompatibilites.</em>
     *
     * @param thisString the <code>String</code> to decode
     * @param thisEncoding   The name of a supported
     *    <a href="../lang/package-summary.html#charenc">character
     *    encoding</a>.
     * @return the newly decoded <code>String</code>
     * @exception  UnsupportedEncodingException
     *             If the named encoding is not supported
     * @see URLEncoder#encode(java.lang.String, java.lang.String)
     * @since 1.4
     */
    public static String decode(String thisString, String thisEncoding) throws UnsupportedEncodingException {
        boolean needToChange = false;
        StringBuffer thisBuffer = new StringBuffer();
        int thisLength = thisString.length();
        int i = 0;

        if (thisEncoding.length() == 0) {
            throw new UnsupportedEncodingException("URLDecoder: empty string enc parameter");
        }

        while (i < thisLength) {
            char c = thisString.charAt(i);
            switch (c) {
                case '+':
                    thisBuffer.append(' ');
                    i++;
                    needToChange = true;
                    break;
                case '%':

                    /*
                     * Starting with this instance of %, process all
                     * consecutive substrings of the form %xy. Each
                     * substring %xy will yield a byte. Convert all
                     * consecutive  bytes obtained this way to whatever
                     * character(s) they represent in the provided
                     * encoding.
                     */
                    try {
                        // (thisLength-i)/3 is an upper bound for the number of remaining bytes
                        byte[] theseBytes = new byte[(thisLength - i) / 3];
                        int thisPos = 0;

                        while (((i + 2) < thisLength) && (c == '%')) {
                            theseBytes[thisPos++] = (byte) Integer.parseInt(thisString.substring(i + 1, i + 3), 16);
                            i += 3;
                            if (i < thisLength) {
                                c = thisString.charAt(i);
                            }
                        }

                        // a trailing, incomplete byte encoding such as "%x" will cause an exception to be thrown
                        if ((i < thisLength) && (c == '%')) {
                            throw new IllegalArgumentException("URLDecoder: Incomplete trailing escape (%) pattern");
                        }

                        thisBuffer.append(new String(theseBytes, 0, thisPos, thisEncoding));
                    } catch (NumberFormatException e) {
                        throw new IllegalArgumentException("URLDecoder: Illegal hex characters in escape (%) pattern - " + e.getMessage());
                    }
                    needToChange = true;
                    break;
                default:
                    thisBuffer.append(c);
                    i++;
                    break;
            }
        }
        return (needToChange ? thisBuffer.toString() : thisString);
    }
}
